@@ -22,6 +22,7 @@ from typing import Optional
 from gi.repository import GLib
 
 from nbxmpp.namespaces import Namespace
+from nbxmpp.http import HTTPSession
 from nbxmpp.protocol import Features
 from nbxmpp.protocol import StanzaMalformed
 from nbxmpp.protocol import SessionRequest
@@ -93,6 +94,7 @@ class Client(Observable):
         self._client_cert = None
         self._client_cert_pass = None
         self._proxy = None
+        self._http_session = HTTPSession()
 
         self._allowed_con_types = None
         self._allowed_protocols = None
@@ -232,6 +234,8 @@ class Client(Observable):
 
     @property
     def peer_certificate(self):
+        if self._con is not None:
+            return self._con.peer_certificate
         return self._peer_certificate, self._peer_certificate_errors
 
     @property
@@ -327,12 +331,18 @@ class Client(Observable):
 
     def set_proxy(self, proxy):
         self._proxy = proxy
-        self._dispatcher.get_module('Muclumbus').set_proxy(proxy)
 
     @property
     def proxy(self):
         return self._proxy
 
+    def set_http_session(self, session: HTTPSession) -> None:
+        self._http_session = session
+
+    @property
+    def http_session(self) -> HTTPSession:
+        return self._http_session
+
     def get_bound_jid(self):
         return self._jid
 
@@ -360,6 +370,8 @@ class Client(Observable):
             return
 
         self.state = StreamState.CONNECTING
+        self._peer_certificate = None
+        self._peer_certificate_errors = None
         self._reset_error()
 
         self._con = self._get_connection(self._log_context,
@@ -375,7 +387,6 @@ class Client(Observable):
         self._con.subscribe('data-sent', self._on_data_sent)
         self._con.subscribe('data-received', self._on_data_received)
         self._con.subscribe('bad-certificate', self._on_bad_certificate)
-        self._con.subscribe('certificate-set', self._on_certificate_set)
         self._con.connect()
 
     def _get_connection(self, *args):
@@ -399,6 +410,7 @@ class Client(Observable):
         self.state = StreamState.RESOLVE
 
         self._addresses = ServerAddresses(self._domain)
+        self._addresses.set_http_session(self._http_session)
         self._addresses.set_custom_host(self._custom_host)
         self._addresses.set_proxy(self._proxy)
         self._addresses.subscribe('resolved', self._on_addresses_resolved)
@@ -537,12 +549,9 @@ class Client(Observable):
             connection.peer_certificate
         self._set_error(StreamError.BAD_CERTIFICATE, 'bad certificate')
 
-    def _on_certificate_set(self, connection, _signal_name):
-        self._peer_certificate, self._peer_certificate_errors = \
-            connection.peer_certificate
-
     def accept_certificate(self):
         self._log.info('Certificate accepted')
+        assert self._peer_certificate is not None
         self._accepted_certificates.append(self._peer_certificate)
         self._connect()
 
