@@ -21,6 +21,7 @@ sub- stanzas) handling routines
 from __future__ import annotations
 
 from typing import Any
+from typing import Iterable
 from typing import Union
 from typing import Optional
 from typing import cast
@@ -37,6 +38,12 @@ from dataclasses import asdict
 from gi.repository import GLib
 
 import idna
+from nbxmpp.xmppiri import escape_ifragment
+from nbxmpp.xmppiri import escape_inode
+from nbxmpp.xmppiri import escape_ires
+from nbxmpp.xmppiri import escape_ivalue
+from nbxmpp.xmppiri import validate_ikey
+from nbxmpp.xmppiri import validate_querytype
 from nbxmpp.simplexml import Node
 from nbxmpp.namespaces import Namespace
 from nbxmpp.stringprep import nodeprep
@@ -314,6 +321,7 @@ ERR_REDIRECT = 'urn:ietf:params:xml:ns:xmpp-stanzas redirect'
 STREAM_UNSUPPORTED_STANZA_TYPE = 'urn:ietf:params:xml:ns:xmpp-streams unsupported-stanza-type'
 ERR_FORBIDDEN = 'urn:ietf:params:xml:ns:xmpp-stanzas forbidden'
 
+
 def isResultNode(node):
     """
     Return true if the node is a positive reply
@@ -767,6 +775,45 @@ class JID:
             return f'{localpart}@{self.domain}{domain_encoded}'
         return f'{localpart}@{self.domain}/{self.resource}{domain_encoded}'
 
+    def to_iri(self,
+               query: Optional[str | tuple[str, list[tuple[str, str]]]] = None,
+               fragment: Optional[str] = None
+               ) -> str:
+
+        if self.localpart:
+            inode = escape_inode(self.localpart)
+            ipathxmpp = f'{inode}@{self.domain}'
+        else:
+            ipathxmpp = f'{self.domain}'
+
+        if self.resource is not None:
+            ires = escape_ires(self.resource)
+            ipathxmpp = f'{ipathxmpp}/{ires}'
+
+        iri = f'xmpp:{ipathxmpp}'
+
+        if query is not None:
+            if isinstance(query, str):
+                querytype = query
+                queryparams = None
+            else:
+                querytype, queryparams = query
+
+            iquerytype = validate_querytype(querytype)
+            iri += f'?{iquerytype}'
+
+            if queryparams is not None:
+                for ikey, ivalue in queryparams:
+                    ivalue = escape_ivalue(ivalue)
+                    ikey = validate_ikey(ikey)
+                    iri += f';{ikey}={ivalue}'
+
+        if fragment is not None:
+            ifragment = escape_ifragment(fragment)
+            iri += f'#{ifragment}'
+
+        return iri
+
     def copy(self) -> JID:
         deprecation_warning('copy() is not needed, JID is immutable')
         return self
@@ -1257,6 +1304,30 @@ class Message(Protocol):
     def setReceiptReceived(self, id_):
         self.setTag('received', namespace=Namespace.RECEIPTS, attrs={'id': id_})
 
+    def setReply(self,
+                 recipient_jid: str,
+                 reply_to_id: str,
+                 fallback_start: int,
+                 fallback_end: int
+                 ) -> None:
+
+        self.setTag(
+            'reply',
+            namespace=Namespace.REPLY,
+            attrs={
+                'id': reply_to_id,
+                'to': recipient_jid})
+
+        fallback_tag = self.setTag(
+            'fallback',
+            namespace=Namespace.FALLBACK,
+            attrs={'for': Namespace.REPLY})
+        fallback_tag.addChild(
+            'body',
+            attrs={
+                'start': str(fallback_start),
+                'end': str(fallback_end)})
+
     def setOOB(self, url, desc=None):
         oob = self.setTag('x', namespace=Namespace.X_OOB)
         oob.setTagData('url', url)
@@ -1272,6 +1343,27 @@ class Message(Protocol):
     def setHint(self, hint):
         self.setTag(hint, namespace=Namespace.HINTS)
 
+    def setReactions(self, target_id: str, emojis: Iterable[str]):
+        reactions = self.addChild(
+            'reactions', namespace=Namespace.REACTIONS, attrs={"id": target_id})
+        for e in emojis:
+            reactions.addChild(
+                'reaction', namespace=Namespace.REACTIONS, payload=[e])
+
+    def getReactions(self) -> Optional[tuple[str, set[str]]]:
+        reactions = self.getTag('reactions', namespace=Namespace.REACTIONS)
+        if not reactions:
+            return None
+
+        react_to = reactions.getAttr('id')
+        if not react_to:
+            return None
+
+        tags = reactions.getTags('reaction', namespace=Namespace.REACTIONS)
+
+        # strip() in case clients surround emojis with whitespace
+        return react_to, {t.getData().strip() for t in tags}
+
 
 class Presence(Protocol):
 
