@@ -30,8 +30,8 @@ from dataclasses import dataclass
 from dataclasses import field
 from datetime import datetime
 
-from gi.repository import Soup
 from gi.repository import Gio
+from gi.repository import GLib
 
 from nbxmpp.simplexml import Node
 from nbxmpp.namespaces import Namespace
@@ -226,7 +226,7 @@ class PGPKeyMetadata(NamedTuple):
 
 
 class OMEMOMessage(NamedTuple):
-    sid: str
+    sid: int
     iv: bytes
     keys: dict[int, tuple[bytes, bool]]
     payload: bytes
@@ -290,6 +290,13 @@ class CorrectionData(NamedTuple):
     id: str
 
 
+class ReplyData(NamedTuple):
+    to: str
+    id: str
+    fallback_start: Optional[int]
+    fallback_end: Optional[int]
+
+
 class ModerationData(NamedTuple):
     stanza_id: str
     moderator_jid: str
@@ -708,8 +715,9 @@ class ProxyData(NamedTuple):
 
     def get_uri(self) -> str:
         if self.username is not None:
-            user_pass = Soup.uri_encode('%s:%s' % (self.username,
-                                                   self.password))
+            username = GLib.uri_escape_string(self.username, None, False)
+            password = GLib.uri_escape_string(self.password, None, False)
+            user_pass = f'{username}:{password}'
             return '%s://%s@%s' % (self.type,
                                    user_pass,
                                    self.host)
@@ -724,6 +732,8 @@ class OMEMOBundle(NamedTuple):
     spk_signature: bytes
     ik: bytes
     otpks: list[dict[str, str]]
+    device_id: int = -1
+    namespace: str = Namespace.OMEMO_TEMP
 
     def pick_prekey(self) -> dict[str, str]:
         return random.SystemRandom().choice(self.otpks)
@@ -746,6 +756,11 @@ class ChatMarker(NamedTuple):
         return self.type == 'acknowledged'
 
 
+class Reactions(NamedTuple):
+    id: str
+    emojis: set[str]
+
+
 class CommonError:
     def __init__(self, stanza):
         self._stanza_name = stanza.getName()
@@ -963,6 +978,7 @@ class MessageProperties:
     http_auth: Optional[HTTPAuthData] = None
     nickname: Optional[str] = None
     from_muc: bool = False
+    occupant_id: Optional[str] = None
     muc_jid: Optional[JID] = None
     muc_nickname: Optional[str] = None
     muc_status_codes: Optional[Set[StatusCode]] = None
@@ -986,12 +1002,14 @@ class MessageProperties:
     receipt: Optional[ReceiptData] = None
     oob: Optional[OOBData] = None
     correction: Optional[CorrectionData] = None
+    reply_data: Optional[ReplyData] = None
     moderation: Optional[ModerationData] = None
     attention: bool = False
     forms = None
     xhtml: Optional[str] = None
     security_label = None
     chatstate = None
+    reactions: Optional[Reactions] = None
 
     def is_from_us(self, bare_match: bool = True):
         if self.from_ is None:
@@ -1183,6 +1201,7 @@ class PresenceProperties:
     self_presence: bool = False
     self_bare: bool = False
     from_muc: bool = False
+    occupant_id: Optional[str] = None
     status: str = ''
     timestamp: float = field(default_factory=time.time)
     user_timestamp: Optional[float] = None
